@@ -22,6 +22,7 @@ package org.apache.sysds.runtime.controlprogram.context;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.io.PrintStream;
+import java.lang.annotation.Target;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
@@ -60,6 +61,9 @@ import org.apache.sysds.conf.DMLConfig;
 import org.apache.sysds.hops.OptimizerUtils;
 import org.apache.sysds.lops.Checkpoint;
 import org.apache.sysds.runtime.DMLRuntimeException;
+import org.apache.sysds.runtime.compress.CompressedMatrixBlock;
+import org.apache.sysds.runtime.compress.io.CompressUnwrap;
+import org.apache.sysds.runtime.compress.io.CompressedWriteBlock;
 import org.apache.sysds.runtime.controlprogram.Program;
 import org.apache.sysds.runtime.controlprogram.caching.CacheBlock;
 import org.apache.sysds.runtime.controlprogram.caching.CacheableData;
@@ -70,6 +74,7 @@ import org.apache.sysds.runtime.controlprogram.parfor.stat.InfrastructureAnalyze
 import org.apache.sysds.runtime.data.SparseBlock;
 import org.apache.sysds.runtime.data.TensorBlock;
 import org.apache.sysds.runtime.data.TensorIndexes;
+import org.apache.sysds.runtime.frame.data.FrameBlock;
 import org.apache.sysds.runtime.instructions.cp.Data;
 import org.apache.sysds.runtime.instructions.spark.DeCompressionSPInstruction;
 import org.apache.sysds.runtime.instructions.spark.data.BroadcastObject;
@@ -86,7 +91,6 @@ import org.apache.sysds.runtime.instructions.spark.utils.RDDAggregateUtils;
 import org.apache.sysds.runtime.instructions.spark.utils.SparkUtils;
 import org.apache.sysds.runtime.io.IOUtilFunctions;
 import org.apache.sysds.runtime.io.InputOutputInfo;
-import org.apache.sysds.runtime.matrix.data.FrameBlock;
 import org.apache.sysds.runtime.matrix.data.MatrixBlock;
 import org.apache.sysds.runtime.matrix.data.MatrixCell;
 import org.apache.sysds.runtime.matrix.data.MatrixIndexes;
@@ -200,11 +204,21 @@ public class SparkExecutionContext extends ExecutionContext
 		return LAZY_SPARKCTX_CREATION;
 	}
 
-	private synchronized static void initSparkContext()
-	{
+	public static void handleIllegalReflectiveAccessSpark(){
+		Module pf = org.apache.spark.unsafe.Platform.class.getModule();
+		Target.class.getModule().addOpens("java.nio", pf);
+
+		Module se = org.apache.spark.util.SizeEstimator.class.getModule();
+		Target.class.getModule().addOpens("java.util", se);
+		Target.class.getModule().addOpens("java.lang", se);
+		Target.class.getModule().addOpens("java.util.concurrent", se);
+	}
+
+	private synchronized static void initSparkContext(){
 		//check for redundant spark context init
 		if( _spctx != null )
 			return;
+		handleIllegalReflectiveAccessSpark();
 
 		long t0 = DMLScript.STATISTICS ? System.nanoTime() : 0;
 
@@ -409,13 +423,13 @@ public class SparkExecutionContext extends ExecutionContext
 			rdd = mo.getRDDHandle().getRDD();
 		}
 		//CASE 2: dirty in memory data or cached result of rdd operations
-		else if( mo.isDirty() || mo.isCached(false) || mo.isFederated() )
+		else if( mo.isDirty() || mo.isCached(false) || mo.isFederated() || mo instanceof MatrixObjectFuture)
 		{
 			//get in-memory matrix block and parallelize it
 			//w/ guarded parallelize (fallback to export, rdd from file if too large)
 			DataCharacteristics dc = mo.getDataCharacteristics();
 			boolean fromFile = false;
-			if( !mo.isFederated() && (!OptimizerUtils.checkSparkCollectMemoryBudget(dc, 0)
+			if( !mo.isFederated() && !(mo instanceof MatrixObjectFuture) && (!OptimizerUtils.checkSparkCollectMemoryBudget(dc, 0)
 				|| !_parRDDs.reserve(OptimizerUtils.estimatePartitionedSizeExactSparsity(dc)))) {
 				if( mo.isDirty() || !mo.isHDFSFileExists() ) //write if necessary
 					mo.exportData();
@@ -441,19 +455,19 @@ public class SparkExecutionContext extends ExecutionContext
 		{
 			// parallelize hdfs-resident file
 			// For binary block, these are: SequenceFileInputFormat.class, MatrixIndexes.class, MatrixBlock.class
-			if(fmt == FileFormat.BINARY) {
-				rdd = sc.hadoopFile( mo.getFileName(), inputInfo.inputFormatClass, inputInfo.keyClass, inputInfo.valueClass);
+			rdd = sc.hadoopFile( mo.getFileName(), inputInfo.inputFormatClass, inputInfo.keyClass, inputInfo.valueClass);
+			if(fmt == FileFormat.BINARY) 
 				//note: this copy is still required in Spark 1.4 because spark hands out whatever the inputformat
 				//recordreader returns; the javadoc explicitly recommend to copy all key/value pairs
-				rdd = SparkUtils.copyBinaryBlockMatrix((JavaPairRDD<MatrixIndexes, MatrixBlock>)rdd); //cp is workaround for read bug
-			}
-			else if(fmt.isTextFormat()) {
-				rdd = sc.hadoopFile( mo.getFileName(), inputInfo.inputFormatClass, inputInfo.keyClass, inputInfo.valueClass);
-				rdd = ((JavaPairRDD<LongWritable, Text>)rdd).mapToPair( new CopyTextInputFunction() ); //cp is workaround for read bug
-			}
-			else {
+				// cp is workaround for read bug
+				rdd = SparkUtils.copyBinaryBlockMatrix((JavaPairRDD<MatrixIndexes, MatrixBlock>)rdd); 
+			else if(fmt == FileFormat.COMPRESSED)
+				rdd = ((JavaPairRDD<MatrixIndexes, CompressedWriteBlock>) rdd).mapValues(new CompressUnwrap());
+			else if(fmt.isTextFormat())
+				// cp is workaround for read bug
+				rdd = ((JavaPairRDD<LongWritable, Text>) rdd).mapToPair(new CopyTextInputFunction());
+			else
 				throw new DMLRuntimeException("Incorrect input format in getRDDHandleForVariable");
-			}
 
 			//keep rdd handle for future operations on it
 			RDDObject rddhandle = new RDDObject(rdd);
@@ -619,9 +633,9 @@ public class SparkExecutionContext extends ExecutionContext
 		return rdd;
 	}
 
-	public Broadcast<CacheBlock> broadcastVariable(CacheableData<CacheBlock> cd) {
+	public Broadcast<CacheBlock<?>> broadcastVariable(CacheableData<CacheBlock<?>> cd) {
 		long t0 = DMLScript.STATISTICS ? System.nanoTime() : 0;
-		Broadcast<CacheBlock> brBlock = null;
+		Broadcast<CacheBlock<?>> brBlock = null;
 
 		// reuse existing non partitioned broadcast handle
 		if (cd.getBroadcastHandle() != null && cd.getBroadcastHandle().isNonPartitionedBroadcastValid()) {
@@ -636,7 +650,7 @@ public class SparkExecutionContext extends ExecutionContext
 				CacheableData.addBroadcastSize(-cd.getBroadcastHandle().getSize());
 
 			// read the matrix block
-			CacheBlock cb = cd.acquireRead();
+			CacheBlock<?> cb = cd.acquireRead();
 			cd.release();
 
 			// broadcast a non-empty frame whose size is smaller than 2G
@@ -669,11 +683,11 @@ public class SparkExecutionContext extends ExecutionContext
 		//the broadcasts are created (other than in local mode) in order to avoid 
 		//unnecessary memory requirements during the lifetime of this broadcast handle.
 		
-		long t0 = DMLScript.STATISTICS ? System.nanoTime() : 0;
 
 		PartitionedBroadcast<MatrixBlock> bret = null;
 
 		synchronized (mo) {  //synchronize with the async. broadcast thread
+			long t0 = DMLScript.STATISTICS ? System.nanoTime() : 0;
 			//reuse existing broadcast handle
 			if (mo.getBroadcastHandle() != null && mo.getBroadcastHandle().isPartitionedBroadcastValid()) {
 				bret = mo.getBroadcastHandle().getPartitionedBroadcast();
@@ -716,10 +730,10 @@ public class SparkExecutionContext extends ExecutionContext
 					OptimizerUtils.estimatePartitionedSizeExactSparsity(mo.getDataCharacteristics()));
 				CacheableData.addBroadcastSize(mo.getBroadcastHandle().getSize());
 
-				if (DMLScript.STATISTICS) {
-					SparkStatistics.accBroadCastTime(System.nanoTime() - t0);
-					SparkStatistics.incBroadcastCount(1);
-				}
+			}
+			if (DMLScript.STATISTICS) {
+				SparkStatistics.accBroadCastTime(System.nanoTime() - t0);
+				SparkStatistics.incBroadcastCount(1);
 			}
 		}
 		return bret;
@@ -851,12 +865,12 @@ public class SparkExecutionContext extends ExecutionContext
 		return bret;
 	}
 	
-	private Broadcast<PartitionedBlock<? extends CacheBlock>> createPartitionedBroadcast(
-			PartitionedBlock<? extends CacheBlock> pmb, int numPerPart, int pos) {
+	private Broadcast<PartitionedBlock<? extends CacheBlock<?>>> createPartitionedBroadcast(
+			PartitionedBlock<? extends CacheBlock<?>> pmb, int numPerPart, int pos) {
 		int offset = pos * numPerPart;
 		int numBlks = Math.min(numPerPart, pmb.getNumRowBlocks() * pmb.getNumColumnBlocks() - offset);
-		PartitionedBlock<? extends CacheBlock> tmp = pmb.createPartition(offset, numBlks);
-		Broadcast<PartitionedBlock<? extends CacheBlock>> ret = getSparkContext().broadcast(tmp);
+		PartitionedBlock<? extends CacheBlock<?>> tmp = pmb.createPartition(offset, numBlks);
+		Broadcast<PartitionedBlock<? extends CacheBlock<?>>> ret = getSparkContext().broadcast(tmp);
 		if (!isLocalMaster())
 			tmp.clearBlocks();
 		return ret;
@@ -875,6 +889,11 @@ public class SparkExecutionContext extends ExecutionContext
 		obj.setRDDHandle( rddhandle );
 	}
 
+	public void setRDDHandleForVariable(String varname, RDDObject rddhandle) {
+		CacheableData<?> obj = getCacheableData(varname);
+		obj.setRDDHandle(rddhandle);
+	}
+
 	public static JavaPairRDD<MatrixIndexes,MatrixBlock> toMatrixJavaPairRDD(JavaSparkContext sc, MatrixBlock src, int blen) {
 		return toMatrixJavaPairRDD(sc, src, blen, -1, true);
 	}
@@ -1099,11 +1118,22 @@ public class SparkExecutionContext extends ExecutionContext
 		// post-processing output matrix
 		if(sparse)
 			out.sortSparseRows();
-		out.setNonZeros(aNnz.longValue());
+		if(containsCompressedMatrixBlock(list))
+			// Recompute zeros since compressed does not maintain it perfectly
+			out.recomputeNonZeros();
+		else 
+			out.setNonZeros(aNnz.longValue());
 		out.examSparsity();
 		return out;
 	}
 
+	private static boolean containsCompressedMatrixBlock(List<Tuple2<MatrixIndexes, MatrixBlock>> list) {
+		for(Tuple2<MatrixIndexes, MatrixBlock> t : list)
+			if(t._2() instanceof CompressedMatrixBlock)
+				return true;
+		return false;
+	}
+
 	private static void blockPartitionsToMatrixBlock(List<Tuple2<MatrixIndexes, MatrixBlock>> tuples, MatrixBlock out,
 		LongAdder aNnz, int blen) {
 		if(out.isInSparseFormat())
@@ -1323,8 +1353,7 @@ public class SparkExecutionContext extends ExecutionContext
 			schema = UtilFunctions.nCopies(clen, ValueType.STRING);
 
 		//create output frame block (w/ lazy allocation)
-		FrameBlock out = new FrameBlock(schema);
-		out.ensureAllocatedColumns(rlen);
+		FrameBlock out = new FrameBlock(schema, rlen);
 
 		List<Tuple2<Long,FrameBlock>> list = rdd.collect();
 
@@ -1487,6 +1516,10 @@ public class SparkExecutionContext extends ExecutionContext
 		if( lob.hasBackReference() )
 			return;
 
+		//abort if a RDD is cached locally
+		if (lob.isInLineageCache())
+			return;
+
 		//cleanup current lineage object (from driver/executors)
 		//incl deferred hdfs file removal (only if metadata set by cleanup call)
 		if( lob instanceof RDDObject ) {
@@ -1766,6 +1799,7 @@ public class SparkExecutionContext extends ExecutionContext
 
 		public SparkClusterConfig()
 		{
+			handleIllegalReflectiveAccessSpark();
 			SparkConf sconf = createSystemDSSparkConf();
 			_confOnly = true;
 
