@@ -42,6 +42,7 @@ import org.apache.sysds.runtime.DMLRuntimeException;
 import org.apache.sysds.runtime.instructions.cp.CPOperand;
 import org.apache.sysds.runtime.instructions.spark.AggregateTernarySPInstruction;
 import org.apache.sysds.runtime.instructions.spark.AggregateUnarySPInstruction;
+import org.apache.sysds.runtime.instructions.spark.AggregateUnarySketchSPInstruction;
 import org.apache.sysds.runtime.instructions.spark.AppendGAlignedSPInstruction;
 import org.apache.sysds.runtime.instructions.spark.AppendGSPInstruction;
 import org.apache.sysds.runtime.instructions.spark.AppendMSPInstruction;
@@ -62,6 +63,7 @@ import org.apache.sysds.runtime.instructions.spark.CumulativeOffsetSPInstruction
 import org.apache.sysds.runtime.instructions.spark.DeCompressionSPInstruction;
 import org.apache.sysds.runtime.instructions.spark.DnnSPInstruction;
 import org.apache.sysds.runtime.instructions.spark.IndexingSPInstruction;
+import org.apache.sysds.runtime.instructions.spark.LIBSVMReblockSPInstruction;
 import org.apache.sysds.runtime.instructions.spark.MapmmChainSPInstruction;
 import org.apache.sysds.runtime.instructions.spark.MapmmSPInstruction;
 import org.apache.sysds.runtime.instructions.spark.MatrixReshapeSPInstruction;
@@ -87,7 +89,7 @@ import org.apache.sysds.runtime.instructions.spark.UnaryFrameSPInstruction;
 import org.apache.sysds.runtime.instructions.spark.UnaryMatrixSPInstruction;
 import org.apache.sysds.runtime.instructions.spark.WriteSPInstruction;
 import org.apache.sysds.runtime.instructions.spark.ZipmmSPInstruction;
-import org.apache.sysds.runtime.instructions.spark.LIBSVMReblockSPInstruction;
+
 
 public class SPInstructionParser extends InstructionParser
 {
@@ -110,7 +112,7 @@ public class SPInstructionParser extends InstructionParser
 		String2SPInstructionType.put( "uacvar"  , SPType.AggregateUnary);
 		String2SPInstructionType.put( "uamax"   , SPType.AggregateUnary);
 		String2SPInstructionType.put( "uarmax"  , SPType.AggregateUnary);
-		String2SPInstructionType.put( "uarimax" ,  SPType.AggregateUnary);
+		String2SPInstructionType.put( "uarimax" , SPType.AggregateUnary);
 		String2SPInstructionType.put( "uacmax"  , SPType.AggregateUnary);
 		String2SPInstructionType.put( "uamin"   , SPType.AggregateUnary);
 		String2SPInstructionType.put( "uarmin"  , SPType.AggregateUnary);
@@ -125,6 +127,14 @@ public class SPInstructionParser extends InstructionParser
 		String2SPInstructionType.put( "uatrace" , SPType.AggregateUnary);
 		String2SPInstructionType.put( "uaktrace", SPType.AggregateUnary);
 
+		// Aggregate unary sketch operators
+		String2SPInstructionType.put( "uacd"    , SPType.AggregateUnarySketch);
+		String2SPInstructionType.put( "uacdr"   , SPType.AggregateUnarySketch);
+		String2SPInstructionType.put( "uacdc"   , SPType.AggregateUnarySketch);
+		String2SPInstructionType.put( "uacdap"  , SPType.AggregateUnarySketch);
+		String2SPInstructionType.put( "uacdapr" , SPType.AggregateUnarySketch);
+		String2SPInstructionType.put( "uacdapc" , SPType.AggregateUnarySketch);
+
 		//binary aggregate operators (matrix multiplication operators)
 		String2SPInstructionType.put( "mapmm"      , SPType.MAPMM);
 		String2SPInstructionType.put( "mapmmchain" , SPType.MAPMMCHAIN);
@@ -181,8 +191,10 @@ public class SPInstructionParser extends InstructionParser
 		String2SPInstructionType.put( "map+*",    SPType.Binary);
 		String2SPInstructionType.put( "map-*",    SPType.Binary);
 		String2SPInstructionType.put( "dropInvalidType", SPType.Binary);
+		String2SPInstructionType.put( "freplicate", SPType.Binary);
 		String2SPInstructionType.put( "mapdropInvalidLength", SPType.Binary);
-		String2SPInstructionType.put( "_map", SPType.Binary); // _map refers to the operation map
+		String2SPInstructionType.put( "valueSwap", SPType.Binary);
+		String2SPInstructionType.put( "_map", SPType.Ternary); // _map refers to the operation map
 		// Relational Instruction Opcodes
 		String2SPInstructionType.put( "=="   , SPType.Binary);
 		String2SPInstructionType.put( "!="   , SPType.Binary);
@@ -228,7 +240,8 @@ public class SPInstructionParser extends InstructionParser
 		String2SPInstructionType.put("libsvmrblk", SPType.LIBSVMReblock);
 
 		// Spark-specific instructions
-		String2SPInstructionType.put( Checkpoint.OPCODE, SPType.Checkpoint);
+		String2SPInstructionType.put( Checkpoint.DEFAULT_CP_OPCODE, SPType.Checkpoint);
+		String2SPInstructionType.put( Checkpoint.ASYNC_CP_OPCODE, SPType.Checkpoint);
 		String2SPInstructionType.put( Compression.OPCODE, SPType.Compression);
 		String2SPInstructionType.put( DeCompression.OPCODE, SPType.DeCompression);
 
@@ -262,7 +275,8 @@ public class SPInstructionParser extends InstructionParser
 		String2SPInstructionType.put( "isinf", SPType.Unary);
 
 		// Parameterized Builtin Functions
-		String2SPInstructionType.put( "autoDiff"   , SPType.ParameterizedBuiltin);
+		String2SPInstructionType.put( "autoDiff",       SPType.ParameterizedBuiltin);
+		String2SPInstructionType.put( "contains",       SPType.ParameterizedBuiltin);
 		String2SPInstructionType.put( "groupedagg",     SPType.ParameterizedBuiltin);
 		String2SPInstructionType.put( "mapgroupedagg",  SPType.ParameterizedBuiltin);
 		String2SPInstructionType.put( "rmempty",        SPType.ParameterizedBuiltin);
@@ -387,6 +401,9 @@ public class SPInstructionParser extends InstructionParser
 			case AggregateUnary:
 				return AggregateUnarySPInstruction.parseInstruction(str);
 
+			case AggregateUnarySketch:
+				return AggregateUnarySketchSPInstruction.parseInstruction(str);
+
 			case AggregateTernary:
 				return AggregateTernarySPInstruction.parseInstruction(str);
 
