@@ -21,6 +21,9 @@ package org.apache.sysds.runtime.instructions.spark;
 
 
 import java.util.Iterator;
+import java.util.concurrent.Callable;
+import java.util.concurrent.Executors;
+import java.util.concurrent.Future;
 import java.util.stream.IntStream;
 
 import org.apache.commons.logging.Log;
@@ -30,6 +33,7 @@ import org.apache.spark.api.java.JavaRDD;
 import org.apache.spark.api.java.function.Function;
 import org.apache.spark.api.java.function.PairFlatMapFunction;
 import org.apache.spark.api.java.function.PairFunction;
+import org.apache.sysds.conf.ConfigurationManager;
 import org.apache.sysds.hops.AggBinaryOp.SparkAggType;
 import org.apache.sysds.hops.OptimizerUtils;
 import org.apache.sysds.lops.MapMult;
@@ -46,6 +50,8 @@ import org.apache.sysds.runtime.instructions.spark.data.LazyIterableIterator;
 import org.apache.sysds.runtime.instructions.spark.data.PartitionedBroadcast;
 import org.apache.sysds.runtime.instructions.spark.functions.FilterNonEmptyBlocksFunction;
 import org.apache.sysds.runtime.instructions.spark.utils.RDDAggregateUtils;
+import org.apache.sysds.runtime.lineage.LineageCacheConfig;
+import org.apache.sysds.runtime.lineage.LineageItem;
 import org.apache.sysds.runtime.matrix.data.MatrixBlock;
 import org.apache.sysds.runtime.matrix.data.MatrixIndexes;
 import org.apache.sysds.runtime.matrix.data.OperationsOnMatrixValues;
@@ -54,9 +60,10 @@ import org.apache.sysds.runtime.matrix.operators.AggregateOperator;
 import org.apache.sysds.runtime.matrix.operators.Operator;
 import org.apache.sysds.runtime.meta.DataCharacteristics;
 
+import org.apache.sysds.runtime.util.CommonThreadPool;
 import scala.Tuple2;
 
-public class MapmmSPInstruction extends BinarySPInstruction {
+public class MapmmSPInstruction extends AggregateBinarySPInstruction {
 	private static final Log LOG = LogFactory.getLog(MapmmSPInstruction.class.getName());
 	
 	private CacheType _type = null;
@@ -126,8 +133,8 @@ public class MapmmSPInstruction extends BinarySPInstruction {
 		}
 		
 		//get inputs
-		PartitionedBroadcast<MatrixBlock> in2 = sec.getBroadcastForVariable(bcastVar); 
-		
+		PartitionedBroadcast<MatrixBlock> in2 = sec.getBroadcastForVariable(bcastVar);
+
 		//empty input block filter
 		if( !_outputEmpty )
 			in1 = in1.filter(new FilterNonEmptyBlocksFunction());
@@ -135,12 +142,25 @@ public class MapmmSPInstruction extends BinarySPInstruction {
 		//execute mapmm and aggregation if necessary and put output into symbol table
 		if( _aggtype == SparkAggType.SINGLE_BLOCK )
 		{
-			JavaRDD<MatrixBlock> out = in1.map(new RDDMapMMFunction2(type, in2));
-			MatrixBlock out2 = RDDAggregateUtils.sumStable(out);
-			
-			//put output block into symbol table (no lineage because single block)
-			//this also includes implicit maintenance of matrix characteristics
-			sec.setMatrixOutput(output.getName(), out2);
+			if (ConfigurationManager.isMaxPrallelizeEnabled()) {
+				try {
+					if(CommonThreadPool.triggerRemoteOPsPool == null)
+						CommonThreadPool.triggerRemoteOPsPool = Executors.newCachedThreadPool();
+					RDDMapmmTask task = new  RDDMapmmTask(in1, in2, type);
+					Future<MatrixBlock> future_out = CommonThreadPool.triggerRemoteOPsPool.submit(task);
+					LineageItem li = !LineageCacheConfig.ReuseCacheType.isNone() ? getLineageItem(ec).getValue() : null;
+					sec.setMatrixOutputAndLineage(output.getName(), future_out, li);
+				}
+				catch(Exception ex) { throw new DMLRuntimeException(ex); }
+			}
+			else {
+				JavaRDD<MatrixBlock> out = in1.map(new RDDMapMMFunction2(type, in2));
+				MatrixBlock out2 = RDDAggregateUtils.sumStable(out);
+
+				//put output block into symbol table (no lineage because single block)
+				//this also includes implicit maintenance of matrix characteristics
+				sec.setMatrixOutput(output.getName(), out2);
+			}
 		}
 		else //MULTI_BLOCK or NONE
 		{
@@ -176,6 +196,10 @@ public class MapmmSPInstruction extends BinarySPInstruction {
 		}
 	}
 
+	public SparkAggType getAggType() {
+		return _aggtype;
+	}
+
 	private static boolean preservesPartitioning(DataCharacteristics mcIn, CacheType type )
 	{
 		if( type == CacheType.LEFT )
@@ -439,4 +463,25 @@ public class MapmmSPInstruction extends BinarySPInstruction {
 			}
 		}
 	}
+
+	private static class RDDMapmmTask implements Callable<MatrixBlock>
+	{
+		JavaPairRDD<MatrixIndexes, MatrixBlock> _in1;
+		PartitionedBroadcast<MatrixBlock> _in2;
+		MapMult.CacheType _type;
+
+		RDDMapmmTask(JavaPairRDD<MatrixIndexes, MatrixBlock> in1, PartitionedBroadcast<MatrixBlock> in2, MapMult.CacheType type) {
+			_in1 = in1;
+			_in2 = in2;
+			_type = type;
+		}
+
+		@Override
+		public MatrixBlock call() {
+			//execute mapmm and aggregation if necessary and put output into symbol table
+			JavaRDD<MatrixBlock> out = _in1.map(new RDDMapMMFunction2(_type, _in2));
+			MatrixBlock out2 = RDDAggregateUtils.sumStable(out);
+			return out2;
+		}
+	}
 }
