@@ -42,6 +42,7 @@ import org.apache.sysds.common.Types.FileFormat;
 import org.apache.sysds.common.Types.ValueType;
 import org.apache.sysds.runtime.controlprogram.context.ExecutionContextFactory;
 import org.apache.sysds.runtime.controlprogram.context.SparkExecutionContext;
+import org.apache.sysds.runtime.frame.data.FrameBlock;
 import org.apache.sysds.runtime.instructions.spark.functions.CopyFrameBlockPairFunction;
 import org.apache.sysds.runtime.instructions.spark.utils.FrameRDDConverterUtils;
 import org.apache.sysds.runtime.instructions.spark.utils.FrameRDDConverterUtils.LongFrameToLongWritableFrameFunction;
@@ -56,7 +57,6 @@ import org.apache.sysds.runtime.io.MatrixReader;
 import org.apache.sysds.runtime.io.MatrixReaderFactory;
 import org.apache.sysds.runtime.io.MatrixWriter;
 import org.apache.sysds.runtime.io.MatrixWriterFactory;
-import org.apache.sysds.runtime.matrix.data.FrameBlock;
 import org.apache.sysds.runtime.matrix.data.MatrixBlock;
 import org.apache.sysds.runtime.matrix.data.MatrixIndexes;
 import org.apache.sysds.runtime.meta.MatrixCharacteristics;
@@ -200,7 +200,7 @@ public class FrameConverterTest extends AutomatedTestBase
 		DMLScript.setGlobalExecMode(ExecMode.SPARK);
 		boolean sparkConfigOld = DMLScript.USE_LOCAL_SPARK_CONFIG;
 		DMLScript.USE_LOCAL_SPARK_CONFIG = true;
-
+		setOutputBuffering(true);
 		try
 		{
 			TestConfiguration config = getTestConfiguration(TEST_NAME);
@@ -380,18 +380,17 @@ public class FrameConverterTest extends AutomatedTestBase
 	}
 	
 	private static void verifyFrameMatrixData(FrameBlock frame, MatrixBlock matrix) {
-		for ( int i=0; i<frame.getNumRows(); i++ )
-			for( int j=0; j<frame.getNumColumns(); j++ ) {
-				Object val1 = UtilFunctions.doubleToObject(frame.getSchema()[j],
-								UtilFunctions.objectToDouble(frame.getSchema()[j], frame.get(i, j)));
-				Object val2 = UtilFunctions.doubleToObject(frame.getSchema()[j], matrix.getValue(i, j));
-				if(( UtilFunctions.compareTo(frame.getSchema()[j], val1, val2)) != 0)
-					Assert.fail("Frame value for cell ("+ i + "," + j + ") is " + val1 + 
-							", is not same as matrix value " + val2);
+		for(int i = 0; i < matrix.getNumRows(); i++)
+			for(int j = 0; j < matrix.getNumColumns(); j++) {
+				double v1 = frame.getDouble(i, j);
+				double v2 = matrix.getDouble(i, j);
+				double diff = Math.abs(v2 - v1);
+				if(diff > 0.000001d)
+					Assert.fail("Frame value for cell (" + i + "," + j + ") is " + v1 + ", is not same as matrix " + v2);
 			}
 	}
-	
-	@SuppressWarnings({ "unchecked", "resource", "cast" })
+
+	@SuppressWarnings({ "unchecked"})
 	private static void runConverter(ConvType type, MatrixCharacteristics mc, MatrixCharacteristics mcMatrix,
 		List<ValueType> schema, String fnameIn, String fnameOut) throws IOException
 	{
